# The DeLINEATE Toolbox -- basic info #

This file is just a quick overview -- for details on usage of various parts of the toolbox, see the Markdown files within the `documentation` folder of this repository or online at http://delineate.it

In theory these docs should be kept in sync with the documentation on the website, but you never know. If you discover discrepancies, please let the dev team know. Best way is probably to create an Issue using the Issues page of our Bitbucket repository -- this will require you to sign up for a (free) Bitbucket account.


## INSTALLATION / DEPENDENCIES ##

The installation of our toolbox is pretty straightforward, but fair warning: Installation of the toolboxes that we rely upon could be a minor-to-major pain. (We rely on PyMVPA for conventional MVPA, and Keras for deep learning -- and then Keras, in turn, depends on either a TensorFlow or Theano backend. And if you're doing GPU-accelerated learning, which hopefully you are if you'd like your analyses to complete while we're all still alive, you will also need Nvidia's CUDA and cuDNN tools.)

We hope to have a more comprehensive guide soon, but since we're working with open-source tools that are in active development, it's hard to provide a definitive set of steps that will work for everyone and won't go out of date nearly instantaneously. But basically, if you have a setup that is capable of running PyMVPA (http://www.pymvpa.org) and/or Keras (https://keras.io), then you should be able to run our stuff without any additional setup. Those toolboxes' documentation is not amazingly detailed but with a little tech-savviness, plus Google and perseverance, you should be able to get it all working. If you have real troubles, the dev team is happy to help people with their own setups -- all we ask in exchange is your help in fleshing out the documentation, so we can pay it forward to future users.

If it helps at all, we have some super-janky and probably somewhat out-of-date (possibly even straight-up incorrect) notes from our initial attempts to set up our own deep learning workstations. Feel free to crib from there as helpful, but take it all with a huge grain of salt. We hope to migrate the bits from that page that are actually useful and true to this documentation eventually, but for now... caveat emptor: http://matthewrjohnson.net/lab/rapwiki/DeepLearningSetup

See the next section for more specific deets on the kinds of hardware/software you're gonna need to get this stuff all working.


## COMPATIBILITY ##

### Hardware and OS ###

**Overview.** Minimally, just about any computer with a working Python installation will do if you want to try out this toolbox. PyMVPA does not do GPU acceleration at the moment, and Keras will run its analyses on the CPU(s) if it doesn't find a compatible GPU. However, most of the value of this toolbox probably comes from doing GPU-accelerated deep learning analyses. You'll probably want reasonable computing hardware to get decent speed out of whatever you're doing, but most of the other stuff doesn't matter nearly as much as the GPU. So if you're trying to allocate resources on a budget, sink more money into the GPU and less into maxing out the other stuff. A decent amount of system memory is a good idea (16 GB would be OK, 32 GB even better... basically you want RAM equivalent in size to the largest chunk of data you'd like to analyze at once, plus a few GB of headroom for the OS and applications). SSD vs conventional hard drive doesn't matter that much beyond the usual speedup to your system overall... most of the time you'll be loading the data once and then analyzing it for a long time thereafter, so disk reading/writing time will not be most people's speed bottleneck. CPU cores mainly only matter if you want to run Keras (deep learning) analyses without GPU acceleration (which even with many CPUs is going to be much slower than using a GPU, so not super-recommended), or if you want to run a bunch of PyMVPA processes simultaneously. So for most people, a run-of-the-mill CPU with 4 medium-fast cores is fine and will not be your main speed bottleneck either. Again, it all depends on your use case but as long as you have a medium-powerful computer in conventional terms, the main thing to think about will be the GPU.

**GPU.** At time of writing, the deep learning libraries that DeLINEATE depends upon only do GPU acceleration for Nvidia GPUs. So that's easy. For most people, you'll see a huge gain of using any semi-decent GPU versus relying on CPUs, and then relatively modest benefits with increasingly fancy GPUs. A pretty decent gaming GPU (e.g. a GeForce GTX 1060, currently retailing for ~300-400 USD) should be sufficient for many cases. You can go on up the line if you like (the GTX 1070 and 1080 line are also nice GPUs), but for the increase in price, you could be better off building more systems with cheaper GPUs if you want to get more analyses run faster, rather than maxing out each GPU. Again, your mileage may vary. If you're trying to decide, the dev team is happy to consult in exchange for your help fleshing out the documentation and benchmark data of this toolbox.

One key note is that even if there are not huge speed bumps between GPU models, they are also differentiated by the amount of onboard GPU RAM. This makes a difference because some Keras models occupy a LOT of GPU RAM. So if you expect to run a complex neural network model and be able to push large chunks of data through it, you may need to shell out for a nicer GPU purely for RAM purposes, even if the speed boost is not very important to you.

### A personal note from the devs ###

OK, full disclosure. We are pushing to have a public release out before a grant deadline and ALL of the verbiage of the documentation is probably not going to happen before that deadline. We are still actively developing the code and docs either way, but this is kinda the stopping point for writing the big, long, primary README file for now. Stuff below is mostly notes-to-ourselves.

Ditto the stuff in the `documentation` directory and on the website... a lot of it is still in primordial format but if you happen upon this toolbox before those primordial forms get overwritten with more content, please just bear with us. You are welcome to contact the dev team with questions and that will help motivate us regarding which parts should get fleshed out first.


### Code libraries and toolboxen we depend on ###

**Overview.** In general, we have tried to minimize dependencies as much as possible. As noted above, we currently have two back-end toolboxes that we support -- PyMVPA for conventional MVPA and Keras for deep learning (with Keras, in turn, relying on either a Theano or TensorFlow back-back-end). You do not need to have both of these installed, and we do not install either one of them for you -- but as long as the one you are intending to use is available on the Python path when you use our toolbox, you should be good to go.

**Architecture.** We use a pretty flexible and bare-bones wrapper architecture that we hope will allow us to maintain compatibility with a pretty wide range of Python/Keras/PyMVPA releases. At the moment, most pieces should work with both Python 2 and 3, and most recent-ish versions of Keras/PyMVPA (as far as we know, though of course we have not tested every possible permutation).

Our advice to you, however, is that you pick a Python version / environment / etc. that works for you, and stick to it as well as you can. Getting and running our toolbox itself is easy -- just download it (either a release version in .zip form or the ongoing development version via our Git repository), and run it from wherever you like (most typically via the delineate.py invocation script). However, our backends and their own dependencies may be a little more finicky.

You can install everything on top of your system default Python if you like, but one thing we have found helpful is to download the Anaconda environment (TK: INSERT LINK) and use that to set up a little sandboxed environment to keep your DeLINEATE stuff in. This is useful just in case things start to get all freaky-deaky and you don't want to disrupt the delicate balance of the rest of your Python ecosystem (assuming you use Python for other things, or intend to someday). We'll leave the details of setting up Anaconda environments to that project's documentation for now, but if anyone is interested in seeing (or better yet, writing) a more complete walkthrough, feel free to get in touch.

**PyMVPA.** This project was not initially intended primarily as a means of providing an interface to PyMVPA or any other conventional MVPA package, but we found it useful to add PyMVPA support so that we could easily compare classifier performance between deep learning and conventional MVPA. (While ensuring that we keep every other aspect of the pre-processing, cross-validation, and so on the same, except for the actual classifier.) That said, you could totally use it just as a front-end way of running PyMVPA analyses if you find PyMVPA's design as baroque and confusing as we do. (No offense, PyMVPA devs, but your class hierarchy is so abstract we hear the ghost of Immanuel Kant is jealous.)

With that said, our PyMVPA support is *somewhat* limited at present. We currently support support vector machine (SVM) and sparse multinomial logistic regression (SMLR) classifiers within PyMVPA, as those have been the workhorses of our own conventional MVPA work. It is relatively easy to add support for more classifiers although not entirely non-trivial, as each classifier in PyMVPA is kind of unique in terms of how it is invoked and what parameters it takes. If you are interested in adding support for any other classifiers, get in touch with us -- it is on our to-do list but hasn't been a priority yet for the primary devs.

We should emphasize that mainly what we are using from PyMVPA is the classifiers themselves, and not any of PyMVPA's other functionality (e.g. cross-validation schemes and so forth). All of that stuff is done within DeLINEATE. This is a little redundant maybe, but we didn't want to have to rely on PyMVPA's architecture and its limitations because that architecture doesn't really work super-great for our deep learning analyses, and we wanted to keep all the infrastructure except for the actual classification identical between our backends.

**Keras.** This is where we started; Keras is our main backend for deep-learning analyses although it is always conceivable that we could add support for others in the future. We currently support most kinds of Sequential models in Keras -- which does not encompass every neural network architecture anyone might want to use, but it does encompass an awfully large number. Similarly, we don't necessarily support every possible parameter to each kind of layer/function/etc. within Keras because some of them are hard to describe with config files alone (without making the end user write any of their own Python code). But you can still do an awful lot, and if there is something you desperately want to accomplish in Keras but can't, get in touch with us -- it might not be too hard to add.

We should note that when we say something "isn't supported" for Keras, that usually means, "assuming you just want to make configuration files and don't want to write any of your own Python code." If, instead, you want to use DeLINEATE as a library of helper functions and are OK writing some of your own code to go along with it, you can do a lot more. (A hybrid approach would also be possible -- e.g. using a configuration file to create a basic neural network architecture and then tweaking it with your own custom code before actually running it.)

### Python versions (2? 3?) ###

* This list is kind of rough and incomplete for now
* Keras should in theory work with both Python 2 and 3, as of the time of writing? (May 2018)
* Officially, it seems like PyMVPA is Python 2.7 only, but so far we have had good luck with Python 3 in most use cases and have seemingly managed to avoid most of the stuff that would break under Python 3.
* We need to do more extensive testing to determine exactly which versions of everything we maintain compatibility with. Right now we are generally using whatever versions of Nvidia drivers / CUDA / cuDNN / Keras / PyMVPA are current, although we started out writing for Keras 1.x (it is at the time of writing up to 2.x) and if you, for some reason, want to keep running in older Keras 1.x, that will probably still work.
* In our Keras stuff, we have mostly used Theano backend but switching to a TensorFlow backend should work too. If you try it out before we do, feel free to let us know how it went so we can add your expertise to this documentation.


### TERMINOLOGY (major object types, etc.) ###

* Just gonna throw out some terms for now, will flesh out more later
* Deep learning
* MVPA
* Training
* Testing
* Validation
* Cross-validation
* delineate.py
* DTAnalysis
* DTData
* DTJob
* DTModel
* DTOutput
* Job file (aka batch file) 
* JSON
* TSV
* TK: An editing mark meaning "to come" or "fill this in later." You will see this a lot in the code/documentation!
