import numpy as np
import scipy.io as sio


def fsw_sample_flat_loader( files ) :
    data_filename = files[0]
    label_filename = files[1]
    my_samples = np.load(data_filename)
    my_labels = np.load(label_filename)
    my_samples = np.reshape(my_samples, (my_samples.shape[0], my_samples.shape[1]*my_samples.shape[2]) )
    my_sa = {}
    my_sa['class'] = my_labels[:,1]
    my_sa['subject'] = my_labels[:,0]
    return my_samples, my_sa, None


def fsw_sample_nonflat_loader( files ) :
    data_filename = files[0]
    label_filename = files[1]
    my_samples = np.load(data_filename)
    my_labels = np.load(label_filename)
    my_sa = {}
    my_sa['class'] = my_labels[:,1]
    my_sa['subject'] = my_labels[:,0]
    return my_samples, my_sa, None


def fso_sample_flat_loader( files ):
    data_filename = files[0]
    mat_data = sio.loadmat( data_filename )
    my_samples = mat_data['erp_data']
    my_samples = np.reshape(my_samples,(np.shape(my_samples)[0],np.shape(my_samples)[1]*np.shape(my_samples)[2]))
    my_sa = {}
    my_sa['subj'] = mat_data['subjects'].flatten()
    my_sa['category'] = mat_data['categories'].flatten()
    return my_samples, my_sa, None


def fso_sample_nonflat_loader( files ):
    data_filename = files[0]
    mat_data = sio.loadmat( data_filename )
    my_samples = mat_data['erp_data']
    my_sa = {}
    my_sa['subj'] = mat_data['subjects'].flatten()
    my_sa['category'] = mat_data['categories'].flatten()
    return my_samples, my_sa, None


def sample_generic_matfile_loader( files ):
    # n.b. "files" is not really files -- we should probably rename this param eventually
    # this will work for mat-files containing one variable of samples (already properly shaped)
    #   and one variable of SAs (sample attributes) e.g. class labels
    # could expand the functionality of this loader later, but for right now we'll assume 
    #   that the first item in the list is the filename, second is the variable name for the 
    #   samples data, and third item in the list is the variable name for the SAs -- which for 
    #   now we will just stick into a generic SA called "class"
    data_filename = files[0]
    samples_varname = files[1]
    labels_varname = files[2]
    mat_data = sio.loadmat( data_filename )
    my_samples = mat_data[ samples_varname ]
    my_sa = {}
    my_sa['class'] = mat_data[ labels_varname ].flatten()
    
    if len(files) > 3:
        other_sa_varnames = files[3:]
        for this_other_sa in other_sa_varnames:
            my_sa[this_other_sa] = mat_data[ this_other_sa ].flatten()
    
    return my_samples, my_sa, None
